package edu.cmu.paid.authentication;

import java.io.*;
import java.lang.*;
import java.util.*;
import edu.cmu.paid.authentication.User;
import edu.cmu.paid.authentication.Group;
import edu.cmu.paid.authentication.util.*;
import edu.cmu.paid.authentication.Exception.*;


public class TestUser {

  public static void main(String [] args) {
     
    User u1,u2,u3,u1_2;
    byte [] k1,k2,k3,k1_2;

    k1 = ByteConverterWrapper.toByteArray(63101);
    k2 = ByteConverterWrapper.toByteArray(63102);
    k3 = ByteConverterWrapper.toByteArray(63103);
    k1_2 = ByteConverterWrapper.toByteArray(6121012);

    System.out.println("k1 = "+k1);
    u1 = new User(1,k1,1,"Pooja","Saksena");

    u2 = new User(2,k2,2,"Arnaldo","Piccinelli");
    u3 = new User(3,k3,3,"David", "Garmire");
    u1_2 = new User(4,k1_2,4,"Qiang","Rao");


    u1.assignUserGroup(1);
    u2.assignUserGroup(2);
    u3.assignUserGroup(3);
    u1_2.assignUserGroup(1);
    u1_2.assignUserGroup(2);

    // Groups have been instantiated now we need to test
    // what they have for input...

    try {
      if (u1.getUserID() != 1) {
	System.out.println("User ID not set properly during initialization!"+u1.getUserID());
	return;
      }

      if (u2.getDealerID() != 2) {
	System.out.println("Delare ID not set properly during initiailzation! Is: "+u2.getDealerID());
	return;
      }
      
      // Get functions have been tested. Time to do the next step... Checking for access rights...
      for (int i=0; i<10; i++) {
	if (!u3.hasPermission(i)) {
	  System.out.println("u3 does not have permissions for "+i);
	  return;
	}
      
	if (!u1_2.hasPermission(i)) {
	  System.out.println("u1-2 does not have permissions for "+i);
	  return;
	}
      
	// If an even value
	if ((i % 2) == 0) {
	  if(!u1.hasPermission(i)) {
	    System.out.println("u1 does not have permissions for "+i);
	    return;
	  }
	  
	  if(u2.hasPermission(i)) {
	    System.out.println("u2 has permissions to "+i+ "when it should not");
	    return;
	  }
	}
	
	// If an odd value
	if ((i % 2) == 1) {
	  if(!u2.hasPermission(i)) {
	    System.out.println("u2 does not have permissions for "+i);
	    return;
	  }
	
	  if(u1.hasPermission(i)) {
	    System.out.println("u1 has permissions to "+i+ "when it should not");
	    return;
	  }
	}

      }

      System.out.println("Testing the getUserGroups function...");
      Vector grps = u1.getUserGroups();
      System.out.print("User 1 belongs to groups:\n->->->-> ");
      String gr;
      for (int i=0; i< grps.size(); i++) {
	gr = (String) grps.elementAt(i);
	System.out.print(gr+", ");
      }
      System.out.println();
      System.out.println("Check the names added for UI.\n");
      System.out.println("u1.last = "+u1.getLastName()+", u1.first = "+u1.getFirstName());
      System.out.println("u2.last = "+u2.getLastName()+", u2.first = "+u2.getFirstName());
      System.out.println("u3.last = "+u3.getLastName()+", u3.first = "+u3.getFirstName());
      System.out.println("u1_2.last = "+u1_2.getLastName()+", u1_2.first = "+u1_2.getFirstName());

    }
    catch (NoUserException e) {
      System.out.println ("Got a NoUserException when didn't want one.");
      return;
    }



    System.out.println("Now trying to serialize the User object...");

    byte [] useByte = ByteConverterWrapper.toByteArray(u1);
    
    User u1a = (User) ByteConverterWrapper.toObject(useByte);

    try {
      String useFName = u1a.getFirstName();
      String useLName = u1a.getLastName();
      if (!useFName.equals(u1.getFirstName()) || !useLName.equals(u1.getLastName())) {
	System.out.println("User's name is no longer "+u1.getFirstName()+" "+u1.getLastName());
	return;
      }
      for (int i=0; i<10; i++) {
	if ((i % 2) == 1) {
	  if(u1a.hasPermission(i)) {
	    System.out.println("Error: Converted user has permissions to: "+i);
	    return;
	  }
	} else {
	  if (!u1a.hasPermission(i)) {
	    System.out.println("Error: Converted user does not have permissions to; "+i);
	    return;
	  }
	}
      }

      if (u1a.getUserID() != u1.getUserID()) {
	System.out.println("Error, UserID did not get converted properly.");
	return;
      }

      if (u1a.getDealerID() != u1.getDealerID()) {
	System.out.println("Error, DealerID did not get converted properly.");
	return;
      }
      
      System.out.println("Testing the getUserGroups function after conversion...");
      Vector grps = u1a.getUserGroups();
      System.out.print("User 1 belongs to groups:\n->->->-> ");
      String gr;
      for (int i=0; i< grps.size(); i++) {
	gr = (String) grps.elementAt(i);
	System.out.print(gr+", ");
      }
      System.out.println();
    }
    catch (NoUserException e) {
      System.out.println("Caught the NoUserException. Should not have happened.");
      return;
    }

    u1a.setUserID(-1);  // Makes the user invalid...


    try {
      boolean b = u1a.hasPermission(1);
    }
    catch (NoUserException e) {
      System.out.println("Caught the exception we were expecting.");
    }

    // Now we test the reinit of a User object
    u3.setUserID(2);
    u2.updateUser(u3);
    try {
      String name,lst;
      System.out.println("Made it to 0");
      name = u2.getFirstName();
      System.out.println("Made it to 1");
      lst = u2.getLastName();
      System.out.println("Made it to 2");
      if (!name.equals(u3.getFirstName()) || !lst.equals(u3.getLastName())) {
	System.out.println("Names did not copy correctly.");
	return;
      }
      if (u2.getDealerID() != u3.getDealerID() || u2.getUserID() != u3.getUserID()){
	System.out.println("User ID or Dealer ID did not copy correctly.");
	return;
      }
      for (int i=0; i<10; i++) {
	if (!u2.hasPermission(i)) {
	  System.out.println("Copied User does not have access to Group "+i);
	  return;
	}
      }
    }
    catch (NoUserException e) {
      System.out.println("Caught NoUserException when not expected. Failed.");
      return;
    }

      
 
    System.out.println("User Tests have passed.");

    //    System.exit (00);
    return;

  }

}
